# vi: ts=4 expandtab
#
# Copyright (C) 2021 VMware Inc.
#

import os

from . import v1tov2
from . import renderer
from cloudinit import util
from cloudinit import subp
from distutils.util import strtobool
from cloudinit import log as logging


LOG = logging.getLogger(__name__)


class Renderer(renderer.Renderer):
    ''' Renders network information in /etc/systemd/network '''

    def __init__(self, config=None):
        if not config:
            config = {}
        self.resolved_conf = config.get('resolved_conf_fn',
                                        '/etc/systemd/resolved.conf')
        self.network_conf_dir = config.get('network_conf_dir',
                                           '/etc/systemd/network/')
        self.network_fname_fptr = config.get('network_fname_fptr', '')

    def create_network_file(self, content):
        ifindex = 10

        LOG.debug('Setting Networking Config Version 2')

        for k, v in content.items():
            if k == 'version' or k != 'ethernets':
                continue

            for key, val in v.items():
                link = key

                conf = '[Match]\n'
                # Generate [Match] section
                if 'match' in val:
                    match = val['match']

                    for i in match:
                        if i == 'macaddress':
                            conf += 'MACAddress=%s\n\n' % (match['macaddress'])
                        elif i == 'driver':
                            conf += 'Driver=%s\n' % (match['driver'])
                            conf += 'Name=%s\n\n' % (link)
                        elif i == 'name':
                            conf += 'Name=%s\n\n' % (match['name'])
                else:
                    conf += 'Name=%s\n\n' % link

                # Generate [Link] section
                if ('mtu' in val):
                    conf += '[Link]\nMTUBytes=%s\n\n' % (val['mtu'])

                # Generate [Network] section
                conf += '[Network]\n'

                if ('dhcp4' in val and 'dhcp6' in val):
                    if (val['dhcp4'] is True and val['dhcp6'] is True):
                        conf += 'DHCP=yes\n'
                    if (val['dhcp4'] is True and val['dhcp6'] is False):
                        conf += 'DHCP=ipv4\n'
                    if (val['dhcp4'] is False and val['dhcp6'] is True):
                        conf += 'DHCP=ipv6\n'
                    if (val['dhcp4'] is False and val['dhcp6'] is False):
                        conf += 'DHCP=no\n'
                elif ('dhcp4' in val):
                    if val['dhcp4'] is True:
                        conf += 'DHCP=ipv4\n'
                elif ('dhcp6' in val):
                    if (val['dhcp6'] is True):
                        conf += 'DHCP=ipv6\n'

                if ('nameservers' in val):
                    nameservers = val['nameservers']

                    if ('search' in nameservers):
                        s = nameservers['search']
                        conf += 'Domains=%s\n' % ' '.join(s)
                    if ('addresses' in nameservers):
                        s = nameservers['addresses']
                        conf += 'DNS=%s\n' % ' '.join(s)

                # Generate [DHCPv4 & DHCPv6] section
                for item in ['dhcp4domain', 'dhcp6domain']:
                    if item not in val:
                        continue
                    ret = str(val[item]).casefold()
                    try:
                        ret = strtobool(ret)
                        ret = 'yes' if ret else 'no'
                    except ValueError:
                        if ret != 'route':
                            LOG.warning('Invalid dhcp4domain value - %s', ret)
                            ret = 'no'
                    if item == 'dhcp4domain':
                        section = '[DHCPv4]'
                    else:
                        section = '[DHCPv6]'
                    conf += '\n' + section + '\nUseDomains=' + ret + '\n\n'

                # Generate [Address] section
                if ('addresses' in val):
                    for i in val['addresses']:
                        conf += '\n[Address]\nAddress=%s\n' % (i)

                # Generate [Route] section
                if ('gateway4' in val):
                    conf += '\n[Route]\nGateway=%s\n' % (val['gateway4'])

                if ('gateway6' in val):
                    conf += '\n[Route]\nGateway=%s\n' % (val['gateway6'])

                if ('routes' in val):
                    routes = val['routes']

                    for i in routes:
                        conf += '\n[Route]\n'
                        if (i['via']):
                            conf += 'Gateway=%s\n' % (i['via'])
                        if (i['to']):
                            conf += 'Destination=%s\n' % (i['to'])
                        if ('metric' in i and i['metric']):
                            conf += 'Metric=%s\n' % (i['metric'])

                net_fn = self.network_conf_dir + '10-cloud-init-' + link + \
                            '.network'
                util.write_file(net_fn, conf)
                util.chownbyname(net_fn, 'systemd-network', 'systemd-network')

    def render_network_state(self, network_state, templates=None, target=None):
        fpnplan = os.path.join(subp.target_path(target), self.network_conf_dir)
        util.ensure_dir(os.path.dirname(fpnplan))

        content = self._render_content(network_state)
        self.create_network_file(content)

    def _render_content(self, network_state):
        if network_state.version == 2:
            LOG.debug('V2 to V2 passthrough')
            return network_state.config
        if network_state.version == 1:
            LOG.debug('Converting from V1 to V2')
            return v1tov2.v1tov2(network_state.config)

def available(target=None):
    expected = ['systemctl']
    search = ['/usr/bin', '/bin']
    for p in expected:
        if not subp.which(p, search=search, target=target):
            return False
    return True
